@@ -11,8 +11,7 @@
 /* *********** main ************* */
 
 
-int main()
-{
+int main(void) {
   /*
   printf("((( print empty plan )))\n");
   print_plan();
