@@ -28,4 +28,11 @@ typedef enum {
     st_HIGH_GUARD=99
 } internal_stacktype_t;
 
+typedef struct stat_s {
+    size_t stat_push_count;
+    size_t stat_pushback_count;
+    size_t stat_realloc_count;
+} stat_t;
+
+
 #endif
