@@ -11,26 +11,26 @@
 #include "check_renderer.h"
 #include "ctstring.h"
 
-render_type render_engine=render_default;
+static render_type render_engine=render_default;
 static bool is_quiet = false;
 
-void set_renderer_to_quiet() {
+void set_renderer_to_quiet(void) {
   is_quiet=true;
 }
 
-void set_renderer_to_ansi() {
+void set_renderer_to_ansi(void) {
   if (isatty (STDOUT_FILENO)) {
     render_engine=render_ansi;
   }
 }
 
-void set_renderer_to_csv() {
+void set_renderer_to_csv(void) {
     render_engine=render_csv;
 }
 
-bool check_if_quiet(const retmsg_t *startp, short rc_is_valid) {
+bool check_if_quiet(const retmsg_t *startp, rc_tristate_t rc_is_valid) {
     return is_quiet
-           && (0 == rc_is_valid)
+           && (rc_valid == rc_is_valid)
            && (startp->rm_type != rm_file)
            && (startp->rm_type != rm_hard_error)
            && (startp->rm_type != rm_error)
