@@ -14,10 +14,10 @@
 #include "ctstring.h"
 #include <errno.h>
 #include <string.h>
-extern int errno;
 #ifdef HAVE_MMAP
 #include <sys/mman.h>
 #endif
+#include <endian.h>
 
 
 off_t ct_seek(ctiff_t * ctif, off_t pos, int whence) {
@@ -102,22 +102,35 @@ static ssize_t ct_read_intern(ctiff_t * ctif, void *buf, size_t count) {
 ssize_t ct_read8(ctiff_t * ctif, uint8 *buf, size_t byte_count) {
     return ct_read_intern(ctif, buf, byte_count);
 }
+static ssize_t ct_read8s(ctiff_t * ctif, int8 *buf, size_t byte_count) {
+    return ct_read_intern(ctif, buf, byte_count);
+}
 
 ssize_t ct_read16(ctiff_t * ctif, uint16 *buf, size_t word_count) {
     size_t count = word_count * sizeof(uint16);
     ssize_t rdbytes = ct_read_intern(ctif, buf, count);
-            for (ssize_t i = 0; i < word_count; i++) {
-                uint16 * tmpbuf = buf+i;
-                *tmpbuf = htole16( *tmpbuf );
-            }
-            return rdbytes;
+    for (size_t i = 0; i < word_count; i++) {
+        uint16 *tmpbuf = buf + i;
+        *tmpbuf = htole16(*tmpbuf);
+    }
+    return rdbytes;
+}
+static ssize_t ct_read16s(ctiff_t * ctif, int16 *buf, size_t word_count) {
+    size_t count = word_count * sizeof(int16);
+    ssize_t rdbytes = ct_read_intern(ctif, buf, count);
+    for (size_t i = 0; i < word_count; i++) {
+        int16 *tmpbuf = buf + i;
+        /* unsure if this works as expected on negative values */
+        *tmpbuf = (int16) htole16((uint16) *tmpbuf);
+    }
+    return rdbytes;
 }
 
 ssize_t ct_read32(ctiff_t * ctif, uint32 *buf, size_t quad_count) {
     size_t count = quad_count * sizeof(uint32);
     ssize_t rdbytes = ct_read_intern(ctif, buf, count);
     //printf("buf32: %lu (0x%x) quadcount=%lu count=%lu rdbytes=%lu\n", *buf, *buf, quad_count, count, rdbytes);
-    for (ssize_t i = 0; i < quad_count; i++) {
+    for (size_t i = 0; i < quad_count; i++) {
         uint32 * tmpbuf = buf+i;
         *tmpbuf = htole32( *tmpbuf );
     }
@@ -125,40 +138,48 @@ ssize_t ct_read32(ctiff_t * ctif, uint32 *buf, size_t quad_count) {
     return rdbytes;
 }
 
-
-int TIFFGetRawTagListIndex(ctiff_t * ctif, tag_t tag) { /* find n-th entry in IFD for given tag, return -1 if not found */
-  switch (ctif->tagorder) {
-    case has_unsorted_tags:
-    case has_sorted_tags:
-        break;
-    case unknown_tag_order:
-      {
-        ctif->tagorder=has_sorted_tags;
-        uint16 count = get_ifd0_count( ctif);
-        if (0 == count) { return no_index_found; }
-        tag_t last = TIFFGetRawTagListEntry(ctif, 0);
-        ctif->tag_cache[last]=0;
-        int is_unsorted = 0;
-        for (uint16 i= 1; i < count; i++) {
-          tag_t current = TIFFGetRawTagListEntry( ctif, i );
-          is_unsorted +=(last >= current);
-          ctif->tag_cache[current] = i;
-          last = current;
-        }
-        if (is_unsorted > 0) { ctif->tagorder = has_unsorted_tags; }
-      }
-  }
-  return ctif->tag_cache[tag];
+static ssize_t ct_read32s(ctiff_t * ctif, int32 *buf, size_t quad_count) {
+    size_t count = quad_count * sizeof(int32);
+    ssize_t rdbytes = ct_read_intern(ctif, buf, count);
+    //printf("buf32: %lu (0x%x) quadcount=%lu count=%lu rdbytes=%lu\n", *buf, *buf, quad_count, count, rdbytes);
+    for (size_t i = 0; i < quad_count; i++) {
+        int32 * tmpbuf = buf+i;
+        /* unsure if this works as expected on negative values */
+        *tmpbuf = (int32) htole32((uint32) *tmpbuf );
+    }
+    //printf("buf32': %lu (0x%x)\n", *buf, *buf);
+    return rdbytes;
 }
 
+static ssize_t ct_readf(ctiff_t * ctif, float *buf, size_t float_count) {
+    size_t count = float_count * sizeof(float);
+    ssize_t rdbytes = ct_read_intern(ctif, buf, count);
+    return rdbytes;
+}
+
+static ssize_t ct_readd(ctiff_t * ctif, double *buf, size_t double_count) {
+    size_t count = double_count * sizeof(double);
+    ssize_t rdbytes = ct_read_intern(ctif, buf, count);
+    return rdbytes;
+}
+
+int TIFFGetRawTagListIndex(const cifd_t *ifd, tag_t tag) { /* find n-th entry in IFD for given tag, return -1 if not found */
+  return ifd->index_cache_by_tag[tag];
+}
+
+#ifdef __TINYC__
+float fabsf(float f) {
+    return  (f >= 0.0) ? f : -f;
+}
+#endif
 
 //------------------------------------------------------------------------------
-ret_t check_tag_has_fvalue(ctiff_t * ctif, tag_t tag, float value) {
+ret_t check_tag_has_fvalue(ctiff_t * ctif, cifd_t *ifd, tag_t tag, float value) {
   GET_EMPTY_RET(ret);
   TIFP_CHECK( ctif, ret);
   float * valp = NULL;
   uint32 found;
-  ret =TIFFGetFieldRATIONAL(ctif, tag, &valp, &found);
+  ret =TIFFGetFieldRATIONAL(ctif, ifd, tag, &valp, &found);
   if ((1 == found) && (NULL != valp)) {
     float val = * valp;
     if ( fabsf(val - value) < 0.01 ) {
@@ -174,21 +195,20 @@ ret_t check_tag_has_fvalue(ctiff_t * ctif, tag_t tag, float value) {
 }
 
 //------------------------------------------------------------------------------
-ret_t check_tag_has_u16value(ctiff_t * ctif, tag_t tag, uint16 value) {
+ret_t check_tag_has_u16value(ctiff_t * ctif, cifd_t *ifd, tag_t tag, uint16 value) {
   GET_EMPTY_RET(ret);
   TIFP_CHECK( ctif, ret);
 
   uint16 * valp = NULL;
   uint32 found;
-  ret=TIFFGetFieldSHORT(ctif, tag, &valp, &found);
+  ret=TIFFGetFieldSHORT(ctif, ifd, tag, &valp, &found);
   if ((1 == found) && (NULL != valp)) {
     uint16 val  = *valp;
     if ( val == value ) {
       ret.returncode=is_valid;
     } else {
       string_t msg = int2str(val);
-      ret = set_value_found_ret(&ret, msg );
-      ret.returncode = tagerror_value_differs;
+      ret = set_value_found_ret_with_returncode(&ret, msg, tagerror_value_differs);
     }
   }
   free(valp);
@@ -197,21 +217,20 @@ ret_t check_tag_has_u16value(ctiff_t * ctif, tag_t tag, uint16 value) {
 
 
 //------------------------------------------------------------------------------
-ret_t check_tag_has_u32value(ctiff_t * ctif, tag_t tag, uint32 value) {
+ret_t check_tag_has_u32value(ctiff_t * ctif, cifd_t *ifd, tag_t tag, uint32 value) {
   GET_EMPTY_RET(ret);
   TIFP_CHECK( ctif, ret);
 
   uint32 * valp = NULL;
   uint32 found;
-  ret=TIFFGetFieldLONG(ctif, tag, &valp, &found);
+  ret=TIFFGetFieldLONG(ctif, ifd, tag, &valp, &found);
   if ((1 == found) && (NULL != valp)) {
     uint32 val = *valp;
     if ( val == value )  {
       ret.returncode=is_valid;
     } else {
       string_t msg = uint2str(val);
-      ret = set_value_found_ret(&ret, msg );
-      ret.returncode = tagerror_value_differs;
+      ret = set_value_found_ret_with_returncode(&ret, msg, tagerror_value_differs);
     }
   }
   free(valp);
@@ -259,7 +278,7 @@ ret_t parse_header_and_endianess(ctiff_t * ctif) {
 
   uint16 magic2 = magic;
   if (endianness) {
-      TIFFSwabShort( &magic2 ); /*  big endian */
+      magic2 = TIFFSwabShort( magic2 ); /*  big endian */
   }
   ret.returncode = should_not_occur;
   if (magic2 == 42) {
@@ -281,13 +300,69 @@ ret_t parse_header_and_endianess(ctiff_t * ctif) {
 }
     if (magic2==0x5352) { strcat(errmsg, ", but could be an Olympus ORF, see http://libopenraw.freedesktop.org/wiki/Olympus_ORF/");
 }
-    ret = set_value_found_ret(&ret, str(errmsg));
-    ret.returncode = tiff_byteorder_error;
+    ret = set_value_found_ret_with_returncode(&ret, str(errmsg), tiff_byteorder_error);
   }
   assert( ret.returncode != should_not_occur);
   return ret;
 }
 
+void empty_cache(cifd_t * ifd) {
+  /* init with -1 indicating 'not found' */
+  memset(ifd->index_cache_by_tag, -1, sizeof(int)* MAXTAGS);
+  memset(ifd->tag_cache_by_index, 0, sizeof(tag_t)* MAXTAGS);
+}
+
+ret_t get_IFD_at_pos(ctiff_t * ctif, uint32 offset, cifd_t * resulting_ifd) {
+    GET_EMPTY_RET(ret);TIFP_CHECK(ctif, ret);
+    if (ct_seek(ctif, offset, SEEK_SET) != offset) {
+        char msg[VALUESTRLEN];
+        snprintf(msg, VALUESTRLEN, "TIFF Header seek error, seek set to byte %u", offset);
+        ret = set_value_found_ret(&ret, str(msg));
+        ret.returncode = tiff_seek_error_header;
+        return ret;
+    }
+    uint16 count;
+    if (ct_read16(ctif, &count, 1) != 2) {
+        char msg[VALUESTRLEN];
+        snprintf(msg, VALUESTRLEN, "TIFF Header ct_read error2, reading ifd count (2 bytes) at file pos=%u (%x)",
+                 offset, offset);
+        ret = set_value_found_ret(&ret, str(msg));
+        ret.returncode = tiff_read_error_header;
+        return ret;
+    }
+    if (is_byteswapped(ctif)) {
+        count = TIFFSwabShort(count);
+    }
+    resulting_ifd->tag_count = count;
+    resulting_ifd->position = (off_t) offset;
+    resulting_ifd->tagorder = unknown_tag_order;
+    /* init with -1 indicating 'not found' */
+    empty_cache(resulting_ifd);
+    switch (resulting_ifd->tagorder) {
+        case has_unsorted_tags:
+        case has_sorted_tags:
+            break;
+        case unknown_tag_order: {
+            tag_t last_tag = TIFFGetRawTagListEntry(ctif, resulting_ifd, 0);
+            resulting_ifd->index_cache_by_tag[last_tag] = 0;
+            resulting_ifd->tag_cache_by_index[0] = last_tag;
+            int is_unsorted = 0;
+            for (uint16 tagidx = 1; tagidx < count; tagidx++) {
+                tag_t current_tag = TIFFGetRawTagListEntry(ctif, resulting_ifd, tagidx);
+
+                is_unsorted += (last_tag >= current_tag);
+                resulting_ifd->index_cache_by_tag[current_tag] = tagidx;
+                resulting_ifd->tag_cache_by_index[tagidx] = current_tag;
+                last_tag = current_tag;
+            }
+            if (is_unsorted > 0) { resulting_ifd->tagorder = has_unsorted_tags; }
+        }
+    }
+    ret.returncode = is_valid;
+    return ret;
+}
+
+
 ret_t get_first_IFD(ctiff_t * ctif, uint32 * ifd) {
   GET_EMPTY_RET(ret);
   TIFP_CHECK( ctif, ret);
@@ -306,7 +381,7 @@ ret_t get_first_IFD(ctiff_t * ctif, uint32 * ifd) {
     return ret;
   }
   if (isByteSwapped) {
-    TIFFSwabLong (&offset);
+    offset = TIFFSwabLong (offset);
   }
   if (offset <= 7) {
     char msg[VALUESTRLEN];
@@ -315,43 +390,44 @@ ret_t get_first_IFD(ctiff_t * ctif, uint32 * ifd) {
     ret.returncode=tiff_ifd0_offset_must_be_greater_than_eight;
     return ret;
   }
-  ctif->ifd0pos=offset;
-  if (ct_seek(ctif, offset, SEEK_SET) != offset ) {
-    char msg[VALUESTRLEN];
-    snprintf(msg, VALUESTRLEN, "TIFF Header seek error, seek set to byte %u", offset);
-    ret=set_value_found_ret(&ret, str(msg));
-    ret.returncode=tiff_seek_error_header;
-    return ret;
-  }
-  uint16 count;
-  if ( ct_read16( ctif, &count, 1) != 2 ) {
-    char msg[VALUESTRLEN];
-    snprintf(msg, VALUESTRLEN, "TIFF Header ct_read error2, reading ifd0 count (2 bytes) at file pos=%u (%x)", offset, offset);
-    ret=set_value_found_ret(&ret, str(msg));
-    ret.returncode=tiff_read_error_header;
-    return ret;
-  }
-  if (is_byteswapped(ctif)) {
-    TIFFSwabShort(&count);
-  }
-  ctif->ifd0c = count;
+  ret = get_IFD_at_pos(ctif, offset, ctif->ifd0);
   *ifd = offset;
-  ret.returncode=is_valid;
   return ret;
 }
 
+/** reads IDS and try to find index of given tag, returns -1 if not found
+ * @param ctif pointer to tiff structure
+ * @param ifd pointer to IFD
+ * @param tag tag
+ * @return index or -1 if not found
+ */
+static int TIFFFind_Index_of_Tag(ctiff_t * ctif, cifd_t *ifd, tag_t tag) {
+    assert(tag >= MINTAGS);
+    uint16 count = get_ifd_count(ifd);
+
+    if (ifd->tagorder !=  unknown_tag_order) {
+        return ifd->index_cache_by_tag[tag];
+    } else {
+        for (uint16 tagidx = 0; tagidx < count; tagidx++) {
+            if (tag == TIFFGetRawTagListEntry(ctif, ifd, tagidx)) {
+                return tagidx;
+            };
+        };
+    }
+    return no_index_found;
+}
+
 
 /* scans first IDF and returns the n-th tag, if errorneous it returns a value 0 */
-tag_t TIFFGetRawTagListEntry(ctiff_t * ctif, int tagidx ) {
+tag_t TIFFGetRawTagListEntry(ctiff_t * ctif, const cifd_t *ifd, int tagidx) {
     tag_t tag;
-    if (ctif->tagorder != unknown_tag_order) {
-        for (tag = 0; tag + 1 == 0; tag++) {
-            if (ctif->tag_cache[tag] == tagidx) return tag;
-        }
+    if (ifd->tagorder == has_sorted_tags) { // only if sorted, the caches should be used
+        //printf("RETURN FROM CACHE: tagidx=%i -> tag=%u\n", tagidx, ifd->tag_cache_by_index[tagidx]);
+       return ifd->tag_cache_by_index[tagidx];
     }
     get_ifd0_count(ctif); /* return code ignored, used to read TIFF header */
     /* ct_read count of tags (2 Bytes) */
-    off_t adress = (off_t) ctif->ifd0pos + 2 + tagidx * 12;
+    off_t adress = (off_t) ifd->position + 2 + tagidx * 12; /* IFD has 2 bytes for count of tags, plus 12 bytes for each tag */
     if (ct_seek(ctif, adress, SEEK_SET) != adress) { /* IFD0 plus 2byte to get IFD-entries, then nth tag */
         //perror ("TIFF Header ct_seek error, seeking adress from ifd0 for tagid");
         //exit( EXIT_FAILURE );
@@ -363,13 +439,14 @@ tag_t TIFFGetRawTagListEntry(ctiff_t * ctif, int tagidx ) {
         return 0;
     }
     if (is_byteswapped(ctif)) {
-        TIFFSwabShort(&tag);
+        tag = TIFFSwabShort(tag);
     }
     return tag;
 }
 
+/* offset data is pointer to memory of values of type offset_type */
 #define OFFSET_MALLOC(ctread, ctif_p, offsetdata, offset_type, count ) {\
-  size_t size = (size_t) sizeof(offset_type) * count;                   \
+  size_t size = (size_t) sizeof(offset_type) * (count);                   \
   /*printf("OFFSET_MALLOC: count=%lu, size=%lu\n", count, size); */                      \
   if((size_t) (ctif_p)->streamlen < size) {\
     char msg[VALUESTRLEN]; \
@@ -383,7 +460,7 @@ tag_t TIFFGetRawTagListEntry(ctiff_t * ctif, int tagidx ) {
     fprintf( stderr, "could not allocate memory for offset_t\n");\
     exit (EXIT_FAILURE);\
   }\
-  ssize_t result = ctread( ctif_p, offsetdata, count);                   \
+  ssize_t result = ctread( ctif_p, (offsetdata), count);                   \
   /*printf("RESULT=%lu\n", result); */                                                                     \
   if (result == 0) {\
     char msg[VALUESTRLEN]; \
@@ -406,18 +483,18 @@ tag_t TIFFGetRawTagListEntry(ctiff_t * ctif, int tagidx ) {
   }\
 }
 
-void offset_swabshort(const ctiff_t * ctif, uint16 * address, uint16 count) {
+static void offset_swabshort(const ctiff_t * ctif, uint16 * address, uint16 count) {
   if (is_byteswapped(ctif)) {
     for (int i=0; i<count; i++, address++) {
-      TIFFSwabShort( address );
+      *address = TIFFSwabShort(*address);
     }
   }
 }
 
-void offset_swablong(const ctiff_t * ctif, uint32 * address, uint16 count) {
+static void offset_swablong(const ctiff_t * ctif, uint32 * address, uint16 count) {
   if (is_byteswapped(ctif)) {
     for (int i=0; i<count; i++, address++) {
-      TIFFSwabLong( address );
+      *address = TIFFSwabLong(*address);
     }
   }
 }
@@ -460,7 +537,7 @@ ret_t read_offsetdata(ctiff_t * ctif, uint32 address, uint32 count, uint16 datat
       offset_swabshort(ctif, offset_p->data16p, count);
       break;
     case TIFF_SSHORT: /* !16-bit signed integer */
-      OFFSET_MALLOC(ct_read16, ctif, offset_p->datas16p, int16, count)
+      OFFSET_MALLOC(ct_read16s, ctif, offset_p->datas16p, int16, count)
       offset_swabshort(ctif, (uint16 *) offset_p->datas16p, count);
       break;
     case TIFF_LONG: /* 32-bit unsigned integer */
@@ -469,7 +546,7 @@ ret_t read_offsetdata(ctiff_t * ctif, uint32 address, uint32 count, uint16 datat
       offset_swablong(ctif, offset_p->data32p, count);
       break;
     case TIFF_SLONG: /* !32-bit signed integer */
-      OFFSET_MALLOC(ct_read32, ctif, offset_p->datas32p, uint32, count)
+      OFFSET_MALLOC(ct_read32s, ctif, offset_p->datas32p, int32, count)
       offset_swablong(ctif, (uint32 *) offset_p->data32p, count);
       break;
     case TIFF_RATIONAL: /* 64-bit unsigned fraction */
@@ -482,11 +559,11 @@ ret_t read_offsetdata(ctiff_t * ctif, uint32 address, uint32 count, uint16 datat
       exit(EXIT_FAILURE);
     case TIFF_FLOAT: /* !32-bit IEEE floating point */
       assert( sizeof(float) == 4);
-      OFFSET_MALLOC(ct_read32, ctif, offset_p->datafloatp, float, count)
+      OFFSET_MALLOC(ct_readf, ctif, offset_p->datafloatp, float, count)
       break;
     case TIFF_DOUBLE: /* !64-bit IEEE floating point */
       assert( sizeof(double) == 8);
-      OFFSET_MALLOC(ct_read_intern, ctif, offset_p->datadoublep, double, count)
+      OFFSET_MALLOC(ct_readd, ctif, offset_p->datadoublep, double, count)
       break;
     case TIFF_LONG8: /* BigTIFF 64-bit unsigned integer */
     case TIFF_IFD8: /* BigTIFF 64-bit unsigned integer (offset) */
@@ -515,16 +592,16 @@ ret_t read_offsetdata(ctiff_t * ctif, uint32 address, uint32 count, uint16 datat
 }
 
 /* scans first IDF and returns the type of the n-th tag */
-ifd_entry_t TIFFGetRawTagIFDListEntry( ctiff_t * ctif, int tagidx ) {
-  uint16 tagcount = get_ifd0_count( ctif);
+ifd_entry_t TIFFGetRawTagIFDListEntry( ctiff_t * ctif, const cifd_t *ifd, int tagidx ) {
+  uint16 tagcount = get_ifd_count( ifd);
   assert( tagcount > 0);
   int byteswapped = is_byteswapped(ctif);
 #ifdef DEBUG
   printf(" count of tags = %i\n", tagcount);
 #endif
-  ifd_entry_t ifd_entry;
+  ifd_entry_t ifd_entry={0};
   ifd_entry.value_or_offset = is_error;
-  off_t adress=(off_t) ctif->ifd0pos+2+tagidx*12;
+  off_t adress=(off_t) get_ifd_pos(ifd) +2+tagidx*12; /* IFD has 2 bytes for count of tags, plus 12 bytes for each tag */
   if (ct_seek(ctif, adress, SEEK_SET) !=adress) { /* IFD0 plus 2byte to get IFD-entries, then nth tag */
     //perror ("TIFF Header ct_seek error, seeking adress from ifd0 for tagid");
     return ifd_entry;
@@ -535,8 +612,8 @@ ifd_entry_t TIFFGetRawTagIFDListEntry( ctiff_t * ctif, int tagidx ) {
     return ifd_entry;
   }
   if (byteswapped) {
-    TIFFSwabShort(&tagid);
-}
+    tagid = TIFFSwabShort(tagid);
+  }
   // tag type check
   uint16 tagtype;
   if ( ct_read16( ctif, &tagtype, 1) != 2) {
@@ -544,7 +621,7 @@ ifd_entry_t TIFFGetRawTagIFDListEntry( ctiff_t * ctif, int tagidx ) {
     return ifd_entry;
   }
   if (byteswapped) {
-    TIFFSwabShort(&tagtype);
+    tagtype = TIFFSwabShort(tagtype);
   }
   uint32 count;
   if ( ct_read32( ctif, &count, 1) != 4) {
@@ -552,7 +629,7 @@ ifd_entry_t TIFFGetRawTagIFDListEntry( ctiff_t * ctif, int tagidx ) {
     return ifd_entry;
   }
   if (byteswapped) {
-    TIFFSwabLong( &count);
+    count = TIFFSwabLong( count);
   }
 #ifdef DEBUG
   printf("\ntagid=%0x, tagtype=%0x count=%0x\n\n", tagid, tagtype, count);
@@ -568,8 +645,8 @@ ifd_entry_t TIFFGetRawTagIFDListEntry( ctiff_t * ctif, int tagidx ) {
         return ifd_entry;
       }
       if (byteswapped) {
-        TIFFSwabLong( &value_or_offset);
-}
+        value_or_offset = TIFFSwabLong( value_or_offset);
+      }
       switch( tagtype) {
         case TIFF_BYTE: /* 8-bit unsigned integer */
         case TIFF_ASCII: /* 8-bit bytes w/ last byte null */
@@ -600,13 +677,13 @@ ifd_entry_t TIFFGetRawTagIFDListEntry( ctiff_t * ctif, int tagidx ) {
             ifd_entry.value_or_offset=is_value;
 	    // swap back if byteswapped
 	    if (byteswapped) {
-		    TIFFSwabLong( &value_or_offset);
+		    value_or_offset = TIFFSwabLong( value_or_offset);
 	    }
             uint16 w0 = (uint16) (value_or_offset & 0x0000ffff);
             uint16 w1 = (uint16) ((value_or_offset >> 16) & 0x0000ffff);;
             if (byteswapped) { /* swap correctly 16bit */
-              TIFFSwabShort( &w0 );
-              TIFFSwabShort( &w1 );
+              w0 = TIFFSwabShort( w0 );
+              w1 = TIFFSwabShort( w1 );
             }
             ifd_entry.data16[0] = w0;
             ifd_entry.data16[1] = w1;
@@ -646,17 +723,11 @@ ifd_entry_t TIFFGetRawTagIFDListEntry( ctiff_t * ctif, int tagidx ) {
 }
 
 /*  TODO */
-ifd_entry_t TIFFGetRawIFDEntry( ctiff_t * ctif, tag_t tag) {
-  int tagidx = no_index_found;
-  for (uint16 i= 0; i < get_ifd0_count( ctif ); i++) {
-    if ((tag >= MINTAGS) && (tag == TIFFGetRawTagListEntry( ctif, i ))) {
-      tagidx= i;
-      break;
-    };
-  };
+ifd_entry_t TIFFGetRawIFDEntry( ctiff_t * ctif, cifd_t *ifd, tag_t tag) {
+  int tagidx = TIFFFind_Index_of_Tag(ctif, ifd, tag);
   ifd_entry_t ifd_entry;
   if (tagidx >= 0) {
-    ifd_entry =  TIFFGetRawTagIFDListEntry( ctif, tagidx );
+    ifd_entry =  TIFFGetRawTagIFDListEntry( ctif, ifd, tagidx );
   } else { /* tag not defined */
 	  printf("\ttag %u (%s) was not found, but requested because defined\n", tag, TIFFTagName(tag));
 	  ifd_entry.value_or_offset = is_error;
@@ -665,36 +736,30 @@ ifd_entry_t TIFFGetRawIFDEntry( ctiff_t * ctif, tag_t tag) {
   return ifd_entry;
 }
 /* scans first IDF and returns the type of the n-th tag */
-uint32 TIFFGetRawTagTypeListEntry( ctiff_t * ctif, int tagidx ) {
+uint32 TIFFGetRawTagTypeListEntry( ctiff_t * ctif, cifd_t *ifd, int tagidx ) {
   if (tagidx >= 0) {
-    ifd_entry_t ifd_entry;
-    ifd_entry =  TIFFGetRawTagIFDListEntry( ctif, tagidx );
+    ifd_entry_t ifd_entry = TIFFGetRawTagIFDListEntry(ctif, ifd, tagidx);
     return ifd_entry.datatype;
   } else { /* tag not defined */
     fprintf(stderr, "tagidx should be greater equal 0");
-    exit(EXIT_FAILURE);
+    return TIFF_ERROR;
   }
 }
 
-/* ct_reads the datatype of given tag on specified TIFF,
+
+/** ct_reads the datatype of given tag on specified TIFF,
  * because FieldType of libtiff does not return the true value (because it maps
  * its own datastructure), we need to use this function instead
- * @param tif pointer to TIFF structure
+ * @param ctif pointer to TIFF structure
+ * @param ifd pointer to IFD
  * @param tag tag
  * @return datatype of given tag
  * if tag does not exists the function aborts with an error
  */
-TIFFDataType TIFFGetRawTagType(ctiff_t * ctif, tag_t tag) {
-  assert(tag >= MINTAGS);
-  int tagidx = no_index_found;
-  for (uint16 i= 0; i < get_ifd0_count( ctif ); i++) {
-    if ( tag == TIFFGetRawTagListEntry( ctif, i )) {
-      tagidx= i;
-      break;
-    };
-  };
+TIFFDataType TIFFGetRawTagType(ctiff_t * ctif, cifd_t *ifd, tag_t tag) {
+  int tagidx = TIFFFind_Index_of_Tag(ctif, ifd, tag);
   if (tagidx >= 0) {
-    TIFFDataType datatype =  TIFFGetRawTagTypeListEntry( ctif, tagidx );
+    TIFFDataType datatype =  TIFFGetRawTagTypeListEntry( ctif, ifd, tagidx );
 #ifdef DEBUG
     printf("### datatype=%i \n", datatype);
 #endif
@@ -725,16 +790,15 @@ ctiff_t * initialize_ctif(const char * tiff_file, ct_ioflag_t ioflag) {
   ctif->streamlen = (ssize_t) fsize(tif);
   ctif->streamp = NULL;
   ctif->actual_streamp = NULL;
-  ctif->tagorder=unknown_tag_order;
-  int * restrict tag_cache = ctif->tag_cache;
-  /* init with -1 indicating 'not found' */
-/*
-  for (register unsigned int i= 0; i<MAXTAGS; i++) {
-    tag_cache[i]= -1;
-  }
-*/
-
-  memset(tag_cache, no_index_found, sizeof(int) * MAXTAGS);
+  static cifd_t ifd0 = {
+          .tagorder=unknown_tag_order,
+          .position=0,
+          .tag_count=0,
+          .index_cache_by_tag={-1},
+          .tag_cache_by_index={0},
+  };
+  /* empty_cache(&ifd0); */
+  ctif->ifd0 = &ifd0;
 
 #ifdef HAVE_MMAP
   switch (ioflag) {
@@ -757,8 +821,6 @@ ctiff_t * initialize_ctif(const char * tiff_file, ct_ioflag_t ioflag) {
 #endif
   ctif->ioflag = ioflag;
   ctif->filename = strdup(tiff_file);
-  ctif->ifd0pos= 0;
-  ctif->ifd0c= 0;
   return ctif;
 }
 
@@ -782,14 +844,22 @@ void free_ctif( ctiff_t * ctif) {
 	/* ctif = NULL; */
 }
 
-uint32 get_ifd0_pos(const ctiff_t * ctif ) {
-	assert( NULL != ctif);
-	return ctif->ifd0pos;
+uint32 get_ifd_pos(const cifd_t *ifd ) {
+	return ifd->position;
 }
 
-uint16 get_ifd0_count(const ctiff_t * ctif ) {
-	assert( NULL != ctif);
-	return ctif->ifd0c;
+uint16 get_ifd_count(const cifd_t *ifd) {
+	return ifd->tag_count;
+}
+
+uint32 get_ifd0_pos(const ctiff_t * ctif) {
+    assert( NULL != ctif);
+    return (get_ifd_pos(ctif->ifd0));
+}
+
+uint16 get_ifd0_count(const ctiff_t * ctif) {
+    assert( NULL != ctif);
+    return (get_ifd_count(ctif->ifd0));
 }
 
 inline int is_byteswapped(const ctiff_t * ctif ) {
@@ -810,8 +880,8 @@ uint32 get_next_ifd_pos(ctiff_t * ctif, uint32 actual_pos) {
 	}
 
 	if (ctif->isbyteswapped) {
-		TIFFSwabShort(&count);
-}
+		count = TIFFSwabShort(count);
+    }
 	ct_seek(ctif, 12 * count, SEEK_CUR); /* each IFD entry is 12 bytes long */
 	uint32 offset;
 	if ( ct_read32( ctif, &offset, 1) != 4 ) {
@@ -821,22 +891,21 @@ uint32 get_next_ifd_pos(ctiff_t * ctif, uint32 actual_pos) {
         return 0;
 	}
 	if (ctif->isbyteswapped) {
-		TIFFSwabLong(&offset);
-}
+		offset = TIFFSwabLong(offset);
+    }
 	return offset;
 }
 
-ret_t TIFFGetFieldASCII(ctiff_t * ctif, tag_t tag, char** string_pp, uint32 * countp ) {
+ret_t TIFFGetFieldASCII(ctiff_t * ctif, cifd_t *ifd, tag_t tag, char** string_pp, uint32 * countp ) {
   assert( *string_pp == NULL);
   assert( countp != NULL);
   GET_EMPTY_RET(ret)
-  int tagidx = TIFFGetRawTagListIndex(ctif, tag);
+  int tagidx = TIFFGetRawTagListIndex(ifd, tag);
   if (tagidx >= 0) { /* there exists the tag */
-    ifd_entry_t entry = TIFFGetRawTagIFDListEntry( ctif, tagidx );
+    ifd_entry_t entry = TIFFGetRawTagIFDListEntry( ctif, ifd, tagidx );
     /* assert (entry.datatype == TIFF_ASCII); */
     if (entry.datatype != TIFF_ASCII) {
-      ret = set_value_found_ret_formatted(&ret, "data of type ASCII expected, but got type %s", TIFFTypeName(entry.datatype));
-      ret.returncode = tagerror_unexpected_type_found;
+      ret = set_value_found_ret_formatted_with_returncode(&ret, tagerror_unexpected_type_found, "data of type ASCII expected, but got type %s", TIFFTypeName(entry.datatype));
       return ret;
     }
     *countp = entry.count;
@@ -891,18 +960,17 @@ ret_t TIFFGetFieldASCII(ctiff_t * ctif, tag_t tag, char** string_pp, uint32 * co
   return ret;
 }
 
-ret_t TIFFGetFieldLONG(ctiff_t * ctif, tag_t tag, uint32 ** long_pp, uint32 * countp) {
+ret_t TIFFGetFieldLONG(ctiff_t * ctif, cifd_t *ifd, tag_t tag, uint32 ** long_pp, uint32 * countp) {
   assert( *long_pp == NULL);
   assert( countp != NULL);
   *countp = 0; /*  init */
   GET_EMPTY_RET(ret)
-  int tagidx = TIFFGetRawTagListIndex(ctif, tag);
+  int tagidx = TIFFGetRawTagListIndex(ifd, tag);
   if (tagidx >= 0) { /* there exists the tag */
-    ifd_entry_t entry = TIFFGetRawTagIFDListEntry( ctif, tagidx );
+    ifd_entry_t entry = TIFFGetRawTagIFDListEntry( ctif, ifd, tagidx );
     uint32 overflow =  (0xffffffff / (sizeof(uint32)));
     if (entry.count > overflow) {
-      ret = set_value_found_ret_formatted(&ret, "count=%u overflow boundary=%u", entry.count, overflow);
-      ret.returncode = tagerror_count_results_in_offsets_overflow;
+      ret = set_value_found_ret_formatted_with_returncode(&ret, tagerror_count_results_in_offsets_overflow, "count=%u overflow boundary=%u", entry.count, overflow);
       return ret;
     }
 
@@ -914,8 +982,7 @@ ret_t TIFFGetFieldLONG(ctiff_t * ctif, tag_t tag, uint32 ** long_pp, uint32 * co
     }
     /* assert (entry.datatype == TIFF_LONG); */
     if (entry.datatype != TIFF_LONG) {
-      ret = set_value_found_ret_formatted(&ret, "data of type LONG expected, but got type %s", TIFFTypeName(entry.datatype));
-      ret.returncode = tagerror_unexpected_type_found;
+      ret = set_value_found_ret_formatted_with_returncode(&ret, tagerror_unexpected_type_found, "data of type LONG expected, but got type %s", TIFFTypeName(entry.datatype));
       return ret;
     }
 
@@ -944,18 +1011,17 @@ ret_t TIFFGetFieldLONG(ctiff_t * ctif, tag_t tag, uint32 ** long_pp, uint32 * co
   return ret;
 }
 
-ret_t TIFFGetFieldSHORT(ctiff_t * ctif, tag_t tag, uint16 ** short_pp, uint32 * countp) {
+ret_t TIFFGetFieldSHORT(ctiff_t * ctif, cifd_t *ifd, tag_t tag, uint16 ** short_pp, uint32 * countp) {
   assert( *short_pp == NULL);
   assert( countp != NULL);
   *countp = 0; /*  init */
   GET_EMPTY_RET(ret)
-  int tagidx = TIFFGetRawTagListIndex(ctif, tag);
+  int tagidx = TIFFGetRawTagListIndex(ifd, tag);
   if (tagidx >= 0) { /* there exists the tag */
-    ifd_entry_t entry = TIFFGetRawTagIFDListEntry( ctif, tagidx );
+    ifd_entry_t entry = TIFFGetRawTagIFDListEntry( ctif, ifd, tagidx );
     uint32 overflow =  (0xffffffff / (sizeof(uint16)));
     if (entry.count > overflow) {
-      ret = set_value_found_ret_formatted(&ret, "count=%u overflow boundary=%u", entry.count, overflow);
-      ret.returncode = tagerror_count_results_in_offsets_overflow;
+      ret = set_value_found_ret_formatted_with_returncode(&ret, tagerror_count_results_in_offsets_overflow, "count=%u overflow boundary=%u", entry.count, overflow);
       return ret;
     }
 
@@ -967,8 +1033,7 @@ ret_t TIFFGetFieldSHORT(ctiff_t * ctif, tag_t tag, uint16 ** short_pp, uint32 *
     }
     /* assert (entry.datatype == TIFF_SHORT); */
     if (entry.datatype != TIFF_SHORT) {
-      ret = set_value_found_ret_formatted(&ret, "data of type SHORT expected, but got type %s", TIFFTypeName(entry.datatype));
-      ret.returncode = tagerror_unexpected_type_found;
+      ret = set_value_found_ret_formatted_with_returncode(&ret, tagerror_unexpected_type_found, "data of type SHORT expected, but got type %s", TIFFTypeName(entry.datatype));
       return ret;
     }
 
@@ -998,18 +1063,17 @@ ret_t TIFFGetFieldSHORT(ctiff_t * ctif, tag_t tag, uint16 ** short_pp, uint32 *
   return ret;
 }
 
-ret_t TIFFGetFieldRATIONAL(ctiff_t * ctif, tag_t tag, float ** float_pp, uint32 * countp) {
+ret_t TIFFGetFieldRATIONAL(ctiff_t * ctif, cifd_t *ifd, tag_t tag, float ** float_pp, uint32 * countp) {
   assert( *float_pp == NULL);
   assert( countp != NULL);
   *countp = 0; /*  init */
   GET_EMPTY_RET(ret)
-  int tagidx = TIFFGetRawTagListIndex(ctif, tag);
+  int tagidx = TIFFGetRawTagListIndex(ifd, tag);
   if (tagidx >= 0) { /* there exists the tag */
-    ifd_entry_t entry = TIFFGetRawTagIFDListEntry( ctif, tagidx );
+    ifd_entry_t entry = TIFFGetRawTagIFDListEntry( ctif, ifd, tagidx );
     uint32 overflow =  (0xffffffff / (2*sizeof(uint32)));
     if (entry.count > overflow) {
-      ret = set_value_found_ret_formatted(&ret, "count=%u overflow boundary=%u", entry.count, overflow);
-      ret.returncode = tagerror_count_results_in_offsets_overflow;
+      ret = set_value_found_ret_formatted_with_returncode(&ret, tagerror_count_results_in_offsets_overflow, "count=%u overflow boundary=%u", entry.count, overflow);
       return ret;
     }
     //printf("entry.count=%i\n", entry.count);
@@ -1021,8 +1085,7 @@ ret_t TIFFGetFieldRATIONAL(ctiff_t * ctif, tag_t tag, float ** float_pp, uint32
     }
     /* assert (entry.datatype == TIFF_RATIONAL); */
     if (entry.datatype != TIFF_RATIONAL) {
-      ret = set_value_found_ret_formatted(&ret, "data of type RATIONAL expected, but got type %s", TIFFTypeName(entry.datatype));
-      ret.returncode = tagerror_unexpected_type_found;
+      ret = set_value_found_ret_formatted_with_returncode(&ret, tagerror_unexpected_type_found, "data of type RATIONAL expected, but got type %s", TIFFTypeName(entry.datatype));
       return ret;
     }
 
@@ -1046,7 +1109,7 @@ ret_t TIFFGetFieldRATIONAL(ctiff_t * ctif, tag_t tag, float ** float_pp, uint32
         uint32 denominator = *(orig_data32p+1);
         //printf("DEBUG: numerator=%u denumeator=%u\n", numerator, denominator);
         float v;
-        if (denominator == 0) {
+        if (0 == denominator) {
           v=NAN;
         } else {
           v = (float) numerator / (float) denominator;
