@@ -16,14 +16,13 @@ ret_t check_all_offsets_are_word_aligned(ctiff_t * ctif) {
   TIFP_CHECK( ctif, ret);
   uint16 count = get_ifd0_count( ctif);
   for (int tagidx = 0; tagidx< count; tagidx++) {
-    ifd_entry_t ifd_entry = TIFFGetRawTagIFDListEntry( ctif, tagidx );
+    ifd_entry_t ifd_entry = TIFFGetRawTagIFDListEntry( ctif, ctif->ifd0, tagidx );
     if (ifd_entry.value_or_offset==is_offset) {
       uint32 offset = ifd_entry.data32offset;
       if ( 0 != (offset & 1)) {
-        uint32 tag = TIFFGetRawTagListEntry( ctif, tagidx);
+        uint32 tag = TIFFGetRawTagListEntry( ctif, ctif->ifd0, tagidx);
         // FIXME: tif_fails?
-        ret = set_value_found_ret_formatted( &ret, "tag %u pointing to 0x%08x and is not word-aligned", tag, offset);
-        ret.returncode = tagerror_offset_not_word_aligned;
+        ret = set_value_found_ret_formatted_with_returncode( &ret, tagerror_offset_not_word_aligned, "tag %u pointing to 0x%08x and is not word-aligned", tag, offset);
         return ret;
       }
     }
