@@ -24,8 +24,7 @@ ret_t check_has_only_one_ifd(ctiff_t * ctif) {
     return ret;
   } else {
     // FIXME: tif_fails?
-      ret = set_value_found_ret_formatted(&ret, "baseline TIFF should have only one IFD, but IFD0 at 0x%08x has pointer to IFDn 0x%08x", offset, IFDn );
-      ret.returncode = ifderror_multiple_ifd_detected;
+      ret = set_value_found_ret_formatted_with_returncode(&ret, ifderror_multiple_ifd_detected, "baseline TIFF should have only one IFD, but IFD0 at 0x%08x has pointer to IFDn 0x%08x", offset, IFDn );
       return ret;
   }
 }
