@@ -22,12 +22,10 @@ ret_t check_cardinality_of_some_tags_are_equal(ctiff_t * ctif) {
   returncode_t rc_stripbytecounts =check_tag_quiet(ctif, TIFFTAG_STRIPBYTECOUNTS);
   if ((rc_stripoffsets == is_valid) && (rc_stripbytecounts == is_valid)) {
       /* find stripoffset */
-      ifd_entry_t stripoffsets_entry = TIFFGetRawIFDEntry(ctif, TIFFTAG_STRIPOFFSETS);
-      ifd_entry_t stripbytescounts_entry = TIFFGetRawIFDEntry(ctif, TIFFTAG_STRIPBYTECOUNTS);
+      ifd_entry_t stripoffsets_entry = TIFFGetRawIFDEntry(ctif, ctif->ifd0, TIFFTAG_STRIPOFFSETS);
+      ifd_entry_t stripbytescounts_entry = TIFFGetRawIFDEntry(ctif, ctif->ifd0, TIFFTAG_STRIPBYTECOUNTS);
       if (stripoffsets_entry.count != stripbytescounts_entry.count) {
-          ret = set_value_found_ret_formatted(&ret, "tag %u count=%u, tag %u count=%u", TIFFTAG_STRIPOFFSETS,
-                   stripoffsets_entry.count, TIFFTAG_STRIPBYTECOUNTS, stripbytescounts_entry.count);
-          ret.returncode = ifderror_different_cardinality;
+          ret = set_value_found_ret_formatted_with_returncode(&ret, ifderror_different_cardinality, "tag %u count=%u, tag %u count=%u", TIFFTAG_STRIPOFFSETS, stripoffsets_entry.count, TIFFTAG_STRIPBYTECOUNTS, stripbytescounts_entry.count);
           return ret;
       }
   }
