@@ -17,12 +17,13 @@
 #define RENDERMAXSIZE (33554432)
 typedef enum{ render_default, render_ansi, render_csv} render_type;
 typedef enum{ within_valid, within_error, within_file, within_summaryerror, within_harderror } render_context_t;
+typedef enum{ rc_undef, rc_valid, rc_invalid } rc_tristate_t; // tristate: -1 undef, 0 valid, 1 invalid
 
 stringbuf_t renderer ( const retmsg_t * ret );
-void set_renderer_to_ansi();
-void set_renderer_to_quiet();
-void set_renderer_to_csv();
-bool check_if_quiet(const retmsg_t *startp, short rc_is_valid);
+void set_renderer_to_ansi(void);
+void set_renderer_to_quiet(void);
+void set_renderer_to_csv(void);
+bool check_if_quiet(const retmsg_t *startp, rc_tristate_t rc_is_valid);
 stringbuf_t renderer_ansi ( const retmsg_t * ret);
 stringbuf_t renderer_default ( const retmsg_t * ret);
 stringbuf_t renderer_csv ( const retmsg_t * ret);
