@@ -34,7 +34,9 @@ typedef enum {
     fc_all_offsets_are_not_zero,
     fc_all_geotiff_tags_have_same_count_of_values,
     fc_check_cardinality,
-    fc_dummy
+    fc_dummy,
+    fc_all_offsets_within_filesize,
+    fc_all_IFDs_have_tags,
 } function_t;
 
 const char * get_parser_function_description (function_t f);
