@@ -17,5 +17,5 @@
 
 unsigned long atoul (const char * s);
 unsigned int count_multiple_zero_bytes(const char *val, uint32 count);
-char * sprint_string_as_hexdump(const char * val, uint32);
+void sprint_string_as_hexdump(char *hexdump_str, size_t hexdump_len, const char *val, uint32);
 #endif
