@@ -16,11 +16,10 @@
 ret_t check_notag(ctiff_t * ctif, tag_t tag) {
   GET_EMPTY_RET(ret);
   TIFP_CHECK( ctif, ret);
-  if (no_index_found == TIFFGetRawTagListIndex(ctif, tag)) {
+  if (no_index_found == TIFFGetRawTagListIndex(ctif->ifd0, tag)) {
       ret.returncode = is_valid;
   } else {
-    ret = set_value_found_ret_formatted(&ret,  "tag %i exists", tag);
-    ret.returncode=tagerror_not_white_listed;
+    ret = set_value_found_ret_formatted_with_returncode(&ret,  tagerror_not_white_listed, "tag %i exists", tag);
   }
   return ret;
 }
