@@ -15,10 +15,10 @@
 ret_t check_tag(ctiff_t * ctif, tag_t tag) {
     GET_EMPTY_RET(ret);
     TIFP_CHECK(ctif, ret);
-    if (no_index_found < TIFFGetRawTagListIndex(ctif, tag)) {
+    if (no_index_found < TIFFGetRawTagListIndex(ctif->ifd0, tag)) {
         ret.returncode=is_valid;
     } else {
-        ret = set_value_found_ret_formatted( &ret, "tag %i is missing", tag);
+        ret = set_value_found_ret_formatted(&ret, "tag %i is missing", tag);
         ret.returncode=tag_does_not_exist;
     }
     return ret;}
