@@ -13,7 +13,7 @@
 
 /* checks if TIF has a specified tag */
 returncode_t check_tag_quiet(ctiff_t * ctif, tag_t tag) {
-  if (no_index_found != TIFFGetRawTagListIndex(ctif, tag)) {
+  if (no_index_found != TIFFGetRawTagListIndex(ctif->ifd0, tag)) {
       return is_valid;
   } return tag_does_not_exist;
 }
