@@ -15,6 +15,7 @@
 #include "config_parser.h"
 #include "check_helper.h"
 #include "msg_parser.h"
+
 #define PCRE2_CODE_UNIT_WIDTH 8
 #include <pcre2.h> /* lib pcre2, see http://www.pcre.org/ */
 #include <libgen.h>
@@ -77,7 +78,7 @@ void print_val(int i, values_t v) {
 */
 
 /* global vars */
-parser_state_t parser_state;
+static parser_state_t parser_state;
 
 /* redefined YY_INPUT to read from stream */
 void
@@ -110,7 +111,7 @@ static void result_push(const full_res_t r) {
 static void result_push_back(const full_res_t r) {
     full_res_t_push_back( parser_state.result_stack, r);
 }
-static void result_printstack() {
+static void result_printstack(void) {
   printf("=== BEGIN result_printstack\n");
   full_res_t_print_top(parser_state.result_stack);
   printf("=== END result_printstack\n");
@@ -125,7 +126,7 @@ static void i_push (unsigned int i) {
     tmp.value.i = i;
     internal_stack_entry_t_push(parser_state.intermediate_stack, tmp);
 }
-static unsigned int i_pop () {
+static unsigned int i_pop (void) {
     internal_stack_entry_t tmp = internal_stack_entry_t_pop(parser_state.intermediate_stack);
     assert(tmp.type == st_unsigned_int);
     return tmp.value.i;
@@ -137,7 +138,7 @@ static void v_push ( values_t v) {
     values_t_push(parser_state.values_stack, v);
 }
 
-static values_t v_pop () {
+static values_t v_pop (void) {
     return values_t_pop(parser_state.values_stack);
 }
 
@@ -151,22 +152,22 @@ static void r_push (const char * r) {
     internal_stack_entry_t_push(parser_state.intermediate_stack, tmp);
 }
 
-static const char * r_pop () {
+static const char * r_pop (void) {
     internal_stack_entry_t tmp = internal_stack_entry_t_pop(parser_state.intermediate_stack);
     assert(tmp.type == st_regex);
     return tmp.value.regex;
 }
 
 static void exe_push (const exe_entry_t i) { exe_entry_t_push( parser_state.exe_stack, i);}
-static exe_entry_t exe_pop () { return exe_entry_t_pop( parser_state.exe_stack); }
+static exe_entry_t exe_pop (void) { return exe_entry_t_pop( parser_state.exe_stack); }
 
-void exe_printstack_human_readable () {
+void exe_printstack_human_readable (void) {
     printf("\n/* the rules are in stack order, the top comes first */\n\n");
     exe_entry_t_print_top(parser_state.exe_stack);
 }
 
 /*  reduce results */
-static void reduce_results() {
+static void reduce_results(void) {
     full_res_t_stack_t * temporary_results = full_res_t_new();
     //full_res_t_init(temporary_results, __FILE__, __LINE__);
 #ifdef DEBUG
@@ -312,6 +313,8 @@ static ret_t call_exec_function(ctiff_t * ctif, ret_t * retp, const exe_entry_t
 	        case fc_internal_logic_combine_open:    { ret.returncode = parser_logical_combine_open ; break; }
 	        case fc_internal_logic_combine_close:   { ret.returncode = parser_logical_combine_close ; break; }
 	        case fc_all_offsets_are_not_zero:       { ret = check_all_offsets_are_greater_zero(ctif); break;}
+            case fc_all_offsets_within_filesize:    { ret = check_all_offsets_within_filesize(ctif); break;}
+            case fc_all_IFDs_have_tags:             { ret = check_all_IFDs_have_tags(ctif); break;}
             case fc_all_geotiff_tags_have_same_count_of_values: {
                                                           ret = check_all_geotiff_tags(ctif); break;
                                                         }
@@ -447,13 +450,12 @@ static ret_t clean_logical_or_if_precondition_failed (const exe_entry_t * exep,
 void execute_plan (ctiff_t * ctif) {
   /*  iterate other function-stack */
   int precondition_result=is_valid;
-  bool last_run_was_a_precondition=false;
   function_t precondition_function=fc_true;
   int * restrict called_tags = parser_state.called_tags;
-  for (register unsigned int i = MINTAGS; i<MAXTAGS; i++) { /* mark all tags as "unprocessed" */
-    called_tags[i]=false;
-  }
+
+  memset(&called_tags[MINTAGS], false, sizeof(int) * (MAXTAGS-MINTAGS));
   if (!exe_entry_t_is_empty(parser_state.exe_stack)) {
+      bool last_run_was_a_precondition=false;
 #ifdef EXE_DEBUG
       printf("------------------------------------\n");
   //i_printstack();
@@ -515,7 +517,7 @@ void execute_plan (ctiff_t * ctif) {
   for (unsigned int tag = MINTAGS;  tag < MAXTAGS; tag++) {
 
     if (
-            (-1 != TIFFGetRawTagListIndex(ctif,  tag))
+            (-1 != TIFFGetRawTagListIndex(ctif->ifd0,  tag))
             && (false==called_tags[tag])
         ) { // tag found in tiff, but not called
 #ifdef EXE_DEBUG
@@ -537,7 +539,7 @@ void execute_plan (ctiff_t * ctif) {
 
 
 /* function to clean an execution plan */
-void clean_plan () {
+void clean_plan (void) {
 }
 
 
@@ -559,8 +561,8 @@ static tag_t settagref( tag_t tag) {
     return tag;
 }
 /* helper function for parser */
-static tag_t gettag( ) { return parser_state.tag;}
-static int incrlineno() {
+static tag_t gettag(void) { return parser_state.tag;}
+static int incrlineno(void) {
   parser_state.lineno++;
 #ifdef RULE_DEBUG
   printf("##lineno=%i\n", parser_state.lineno);
@@ -569,18 +571,18 @@ static int incrlineno() {
 }
 
 /* helper function for parser */
-static int getlineno() { return parser_state.lineno;}
+static int getlineno(void) { return parser_state.lineno;}
 
 
 /* helper function for parser */
-static void commentline() {
+static void commentline(void) {
 #ifdef RULE_DEBUG
   printf("commentline, %i\n", parser_state.lineno);
 #endif
 }
 
 /* helper function for parser */
-static void set_mandatory() {
+static void set_mandatory(void) {
 #ifdef RULE_DEBUG
   printf("tag '%u' is mandatory\n", gettag());
 #endif
@@ -588,7 +590,7 @@ static void set_mandatory() {
 }
 
 /* helper function for parser */
-static void set_optional() {
+static void set_optional(void) {
 #ifdef RULE_DEBUG
   printf("tag '%u' is optional\n", gettag());
 #endif
@@ -598,7 +600,7 @@ static void set_optional() {
 }
 
 /* helper function for parser */
-static void set_ifdepends() {
+static void set_ifdepends(void) {
 #ifdef RULE_DEBUG
   printf("tag '%u' is set if depends\n", gettag());
 #endif
@@ -606,7 +608,7 @@ static void set_ifdepends() {
 }
 
 /* helper function for parser */
-static void set_optdepends() {
+static void set_optdepends(void) {
 #ifdef RULE_DEBUG
   printf("tag '%u' is set optional depends\n", gettag());
 #endif
@@ -615,16 +617,16 @@ static void set_optdepends() {
 
 /* helper function for parser */
 static void regex_push(char * regex_string) {
-  pcre2_code *re;
   PCRE2_SIZE erroffset;
   int errorcode;
-  re = pcre2_compile(
-      (unsigned char *)  regex_string, /* the pattern */
-      PCRE2_ZERO_TERMINATED,           /* the pattern length in code units */
-      0,                               /* default options */
-      &errorcode,                      /* for error code */
-      &erroffset,                      /* for error offset */
-      NULL);                           /* no compile context */
+  pcre2_code *re = pcre2_compile(
+      (unsigned char *) regex_string, /* the pattern */
+      PCRE2_ZERO_TERMINATED, /* the pattern length in code units */
+      0, /* default options */
+      &errorcode, /* for error code */
+      &erroffset, /* for error offset */
+      NULL
+  );                           /* no compile context */
   if (NULL != re) {
 #ifdef DEBUG
     printf("regex found: '%s' in line %i\n", regex_string, parser_state.lineno);
@@ -640,17 +642,17 @@ static void regex_push(char * regex_string) {
 }
 
 /* helper function for parser */
-static void reset_valuelist() {
+static void reset_valuelist(void) {
   parser_state.valuelist = 0;
 }
 
 /* helper function for parser */
-static void incr_values () {
+static void incr_values (void) {
   parser_state.valuelist++;
 }
 
 /* prepare functions for preconditions */
-static exe_entry_t prepare_internal_entry() {
+static exe_entry_t prepare_internal_entry(void) {
   exe_entry_t p;
   p.stack = internal_stack_entry_t_new();
   p.lineno=getlineno();
@@ -882,14 +884,14 @@ static void evaluate_req_and_push_exe(requirements_t req, exe_entry_t e) {
     }
 }
 
-static void set_rule_logical_open() {
+static void set_rule_logical_open(void) {
 #ifdef RULE_DEBUG
   printf("rule_add_logical_config\n");
 #endif
   parser_state.within_logical_or=true;
 }
 
-static void incr_logical_elements() {
+static void incr_logical_elements(void) {
 #ifdef RULE_DEBUG
   printf("incr logical elements to %i\n",  parser_state.logical_elements+1);
 #endif
@@ -898,7 +900,7 @@ static void incr_logical_elements() {
 }
 
 /* helper function for parser */
-static void set_rule_logical_close() {
+static void set_rule_logical_close(void) {
 #ifdef RULE_DEBUG
   printf("reset_logical_list\n");
 #endif
@@ -916,7 +918,7 @@ static void set_rule_logical_close() {
 
 /* this adds the config of a tagline to execution plan
  * HINT: order of calling arguments from stacks is IMPORTANT! */
-static void rule_addtag_config() {
+static void rule_addtag_config(void) {
   if(!values_t_is_empty(parser_state.values_stack)) {
 #ifdef RULE_DEBUG
     printf("rule_addtag_config\n");
@@ -977,9 +979,13 @@ static void set_mode(modes_t mode) {
                                       exe_push(e);
                                       e.function = fc_all_offsets_are_not_zero;
                                       exe_push(e);
+                                      e.function = fc_all_offsets_within_filesize;
+                                      exe_push(e);
                                       break;
                                     }
     case mode_enable_ifd_checks: {
+                                      e.function = fc_all_IFDs_have_tags;
+                                      exe_push(e);
                                       e.function = fc_all_IFDs_are_word_aligned;
                                       exe_push(e);
                                       e.function = fc_tagorder;
@@ -1006,7 +1012,7 @@ static void set_mode(modes_t mode) {
 
 
 /* reset the parser state */
-void reset_parser_state() {
+static void reset_parser_state(void) {
   parser_state.any_reference=0;
   parser_state.exe_stack = exe_entry_t_new();
   parser_state.values_stack = values_t_new();
@@ -1072,9 +1078,8 @@ static void set_include( const char * include_file ) {
 		exit (EXIT_FAILURE);
 	};
 	clean_plan();
-	yycontext ctx2;
-	memset(&ctx2, 0, sizeof(yycontext));
-	FILE * old_stream = parser_state.stream;
+	yycontext ctx2 = {0};
+    FILE * old_stream = parser_state.stream;
 	int old_lineno = parser_state.lineno;
     const char * old_file = parser_state.cfg_filename;
 	printf("At lineno %i, file '%s', switching to include file '%s' (expanded: '%s')\n", old_lineno, parser_state.cfg_filename, include_file, include_file_expanded);
@@ -1094,10 +1099,9 @@ static void set_include( const char * include_file ) {
 
 
 /* function to parse a config file from STDIN */
-void parse_plan () {
+void parse_plan (void) {
   reset_parser_state();
-  yycontext ctx;
-  memset(&ctx, 0, sizeof(yycontext));
+  yycontext ctx = {0};
   parser_state.stream=stdin;
   parser_state.cfg_filename=NULL;
   while (yyparse(&ctx))     /* repeat until EOF */
@@ -1109,8 +1113,7 @@ void parse_plan () {
 /* function to parse a config file from file stream */
 void parse_plan_via_stream( FILE * file, const char * optional_cfg_file ) {
   reset_parser_state();
-  yycontext ctx;
-  memset(&ctx, 0, sizeof(yycontext));
+  yycontext ctx = {0};
   parser_state.stream=file;
   parser_state.cfg_filename=optional_cfg_file;
   while (yyparse(&ctx))     /* repeat until EOF */
@@ -1134,12 +1137,12 @@ void parse_plan_via_file( const char * cfg_file ) {
  * @param msg describes the details about what was going wrong
  * @param yytext gives context of the error
  */
-noreturn static void set_parse_error(char * msg, char * yytext) {
+static void set_parse_error(char * msg, char * yytext) {
   fprintf(stderr, "%s at line %i (error at '%s')\n", msg, parser_state.lineno, yytext);
   exit(EXIT_FAILURE);
 }
 
-void clean_plan_results() {
+static void clean_plan_results(void) {
     full_res_t_delete(parser_state.result_stack);
 //    free(parser_state.result_stack);
     parser_state.result_stack = full_res_t_new();
